pub enum MixedValue<T: EnumSetType> {
Valid(T),
Invalid(u32),
}Expand description
Used to return potentially invalid variants of a MixedEnumSet.
Variants
Valid(T)
A bit that is valid for the enum type.
Invalid(u32)
A bit that does not correspond to any variant of the enum.