MixedEnumSet

Struct MixedEnumSet 

pub struct MixedEnumSet<T: EnumSetTypeWithRepr> { /* private fields */ }
A variant of EnumSet that preserves unknown bits.

It only works for enums with an #[enumset(repr = "…")] attribute used with a primitive integer type.

Numeric Representation

MixedEnumSet uses the same underlying numeric representation as EnumSet. However, bits that do not correspond to an enum variant can be set.

Serialization

When the serde feature is enabled, MixedEnumSets can be serialized and deserialized using the serde crate. It is always serialized as a single integer of the underlying repr type.

Unlike EnumSet, it ignores all flags given to EnumSetType.

FFI Safety

MixedEnumSet may be used interchangeably with the specified repr type in FFI.

Implementations

Source

impl<T: EnumSetTypeWithRepr> MixedEnumSet<T>

pub const fn new() -> Self

Creates an empty MixedEnumSet.

pub const fn empty() -> Self

Creates an empty MixedEnumSet.

This is an alias for MixedEnumSet::new.

pub const fn all() -> Self

Returns a MixedEnumSet containing all valid variants of the enum.

pub const fn variant_count() -> u32

The number of valid variants that this type can contain.

pub fn only(t: T) -> Self

Returns a set containing a single element.

pub fn bit_index(t: T) -> u32

Returns the bit a given enum variant is stored in.

If this returns n, it means the bit is stored in the n + 1th least significant bit of the underlying integer, corresponding to a mask of 1 << n.

pub fn is_bit_valid(bit: u32) -> bool

Returns whether a given bit is valid for this set.

pub fn len(&self) -> usize

Returns the number of elements in this set.

pub fn is_empty(&self) -> bool

Returns true if the set contains no elements.

pub fn clear(&mut self)

Removes all elements from the set.

pub fn contains(&self, value: T) -> bool

Checks whether this set contains a value.

pub fn insert(&mut self, value: T) -> bool

Adds a value to this set.

If the set did not have this value present, true is returned.

If the set did have this value present, false is returned.

pub fn remove(&mut self, value: T) -> bool

Removes a value from this set. Returns whether the value was present in the set.

pub fn is_disjoint(&self, other: impl Into<Self>) -> bool

Returns true if self has no elements in common with other. This is equivalent to checking for an empty intersection.

pub fn is_superset(&self, other: impl Into<Self>) -> bool

Returns true if the set is a superset of another, i.e., self contains at least all the values in other.

pub fn is_subset(&self, other: impl Into<Self>) -> bool

Returns true if the set is a subset of another, i.e., other contains at least all the values in self.

pub fn union(&self, other: impl Into<Self>) -> Self

Returns a set containing any elements present in either set.

pub fn intersection(&self, other: impl Into<Self>) -> Self

Returns a set containing every element present in both sets.

pub fn difference(&self, other: impl Into<Self>) -> Self

Returns a set containing every element present in self but not in other.

pub fn symmetric_difference(&self, other: impl Into<Self>) -> Self

Returns a set containing every element present in either self or other, but not present in both.

pub fn symmetrical_difference(&self, other: impl Into<Self>) -> Self

👎Deprecated since 1.1.13:

use symmetric_difference instead

Returns a set containing every element present in either self or other, but not present in both.

This is a legacy name for this function, and should not be used.

pub fn complement(&self) -> Self

Returns a set containing all enum variants not in this set.

This method clears any unknown bits already existing in the set.

pub fn full_complement(&self) -> Self

Returns a set containing all bits not in this set.

This method sets any unknown bits not already existing in the set.

pub fn valid_len(&self) -> usize

Returns the number of elements in this set, excluding unknown bits.

pub fn has_unknown_bits(&self) -> bool

Returns whether this bitset contains any bits that do not correspond to a valid variant.

pub fn has_bit(&self, value: u32) -> bool

Checks whether this set contains a specific bit.

pub fn insert_bit(&mut self, value: u32) -> bool

Adds a specific bit to this set.

If the set did not have this bit present, true is returned.

If the set did have this bit present, false is returned.

pub fn remove_bit(&mut self, value: u32) -> bool

Removes a specific bit from this set. Returns whether the bit was present in the set.

pub fn insert_all(&mut self, other: impl Into<Self>)

Adds all elements in another set to this one.

pub fn remove_all(&mut self, other: impl Into<Self>)

Removes all values in another set from this one.

Source

impl<T: EnumSetTypeWithRepr> MixedEnumSet<T>

pub const fn as_repr(&self) -> <T as EnumSetTypeWithRepr>::Repr

Returns a T::Repr representing the elements of this set.

Unlike the other as_* methods, this method is zero-cost and guaranteed not to fail, panic or truncate any bits.

pub fn from_repr(bits: <T as EnumSetTypeWithRepr>::Repr) -> Self

Constructs a bitset from a T::Repr.

pub fn from_repr_truncated(bits: <T as EnumSetTypeWithRepr>::Repr) -> Self

Constructs a bitset from a T::Repr, ignoring invalid variants.

pub fn as_enumset(&self) -> EnumSet<T>

Converts this set into the corresponding EnumSet.

If any unknown bits are present in the set, this method will panic.

pub fn try_as_enumset(&self) -> Option<EnumSet<T>>

Attempts to convert this set into the corresponding EnumSet.

If any unknown bits are present in the set, this method will return None.

pub fn as_enumset_truncate(&self) -> EnumSet<T>

Converts this set into the corresponding EnumSet, ignoring bits that do not correspond to a variant.

Source

impl<T: EnumSetTypeWithRepr> MixedEnumSet<T>

pub fn iter(&self) -> MixedEnumSetIter<T>

Iterates the contents of the set in order from the least significant bit to the most significant bit.

Note that iterator invalidation is impossible as the iterator contains a copy of this type, rather than holding a reference to it.